class Price:
    def __init__(self, amount, currency):
        self.amount = amount
        self.currency = currency

    def __str__(self):
        return f'{self.currency} {self.amount}'

    def __eq__(self, other):
        return self.amount == other.amount and self.currency == other.currency
