# Generated by Django 2.0.6 on 2018-07-01 18:32

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Customer',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Plan',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Subscription',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Event',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('object_id', models.CharField(default=None, max_length=50, null=True)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
                ('content_type', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.SET_DEFAULT, to='contenttypes.ContentType')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Card',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Charge',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Coupon',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Invoice',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='InvoiceItem',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Order',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Product',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SKU',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Source',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Transfer',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='BalanceTransaction',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Dispute',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Refund',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='BankAccount',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Account',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Payout',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Application',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='IssuerFraudRecord',
            fields=[
                ('id', models.CharField(editable=False, max_length=255, primary_key=True, serialize=False)),
                ('api_version', models.CharField(editable=False, max_length=12)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('deleted_at', models.DateTimeField(editable=False, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='RayuresMeta',
            fields=[
                ('id', models.CharField(max_length=255, primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('deleted_at', models.DateTimeField(null=True)),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('idempotency_key', models.TextField(null=True)),
                ('request_id', models.TextField(null=True)),
                ('source', models.TextField(null=True)),
                ('event', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='rayures.Event')),
            ],
        ),
        migrations.CreateModel(
            name='RayureEventProcess',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.CharField(choices=[('received', 'received'), ('success', 'success'), ('failure', 'failure')], default='received', max_length=12)),
                ('started_at', models.DateTimeField(auto_now_add=True)),
                ('ended_at', models.DateTimeField(editable=False, null=True)),
                ('event', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='rayures.Event')),
                ('traces', django.contrib.postgres.fields.jsonb.JSONField(default=list)),
            ],
            options={
                'verbose_name': 'Rayure event process',
                'verbose_name_plural': 'Rayure event processes',
            },
        ),
        migrations.CreateModel(
            name='RayureEventProcessingError',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('http_body', models.TextField()),
                ('func', models.TextField(null=True)),
                ('message', models.TextField()),
                ('traceback', models.TextField()),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('acknowledged_at', models.DateTimeField(null=True)),
                ('process', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='errors', to='rayures.RayureEventProcess')),
            ],
        ),
    ]
